@@ -112,10 +112,10 @@ mod tests {
     use std::path::PathBuf;
 
     use pretty_assertions::assert_eq;
-    use tuirealm::tui::style::Color;
+    use tuirealm::ratatui::style::Color;
 
     use super::*;
-    use crate::config::bookmarks::{Bookmark, S3Params, SmbParams, UserHosts};
+    use crate::config::bookmarks::{Bookmark, KubeParams, S3Params, SmbParams, UserHosts};
     use crate::config::params::UserConfig;
     use crate::config::themes::Theme;
     use crate::filetransfer::FileTransferProtocol;
@@ -366,7 +366,7 @@ mod tests {
         assert_eq!(host.username.as_deref().unwrap(), "root");
         assert_eq!(host.password, None);
         // Verify bookmarks
-        assert_eq!(hosts.bookmarks.len(), 5);
+        assert_eq!(hosts.bookmarks.len(), 6);
         let host: &Bookmark = hosts.bookmarks.get("raspberrypi2").unwrap();
         assert_eq!(host.address.as_deref().unwrap(), "192.168.1.31");
         assert_eq!(host.port.unwrap(), 22);
@@ -404,6 +404,19 @@ mod tests {
         assert_eq!(s3.access_key.as_deref().unwrap(), "pippo");
         assert_eq!(s3.secret_access_key.as_deref().unwrap(), "pluto");
         assert_eq!(s3.new_path_style.unwrap(), true);
+        // Kube pod
+        let host: &Bookmark = hosts.bookmarks.get("pod").unwrap();
+        assert_eq!(host.address, None);
+        assert_eq!(host.port, None);
+        assert_eq!(host.username, None);
+        assert_eq!(host.password, None);
+        assert_eq!(host.protocol, FileTransferProtocol::Kube);
+        let kube = host.kube.as_ref().unwrap();
+        assert_eq!(kube.namespace.as_deref().unwrap(), "my-namespace");
+        assert_eq!(kube.cluster_url.as_deref().unwrap(), "https://my-cluster");
+        assert_eq!(kube.username.as_deref().unwrap(), "my-username");
+        assert_eq!(kube.client_cert.as_deref().unwrap(), "my-cert");
+        assert_eq!(kube.client_key.as_deref().unwrap(), "my-key");
 
         // smb
         let host = hosts.bookmarks.get("smb").unwrap();
@@ -443,6 +456,7 @@ mod tests {
                 password: None,
                 remote_path: None,
                 local_path: None,
+                kube: None,
                 s3: None,
                 smb: None,
             },
@@ -457,6 +471,7 @@ mod tests {
                 password: Some(String::from("password")),
                 remote_path: Some(PathBuf::from("/tmp")),
                 local_path: Some(PathBuf::from("/usr")),
+                kube: None,
                 s3: None,
                 smb: None,
             },
@@ -480,9 +495,33 @@ mod tests {
                     secret_access_key: None,
                     new_path_style: None,
                 }),
+                kube: None,
                 smb: None,
             },
         );
+        // push kube pod
+        bookmarks.insert(
+            String::from("pod"),
+            Bookmark {
+                address: None,
+                port: None,
+                protocol: FileTransferProtocol::Kube,
+                username: None,
+                password: None,
+                remote_path: None,
+                local_path: None,
+                s3: None,
+                smb: None,
+                kube: Some(KubeParams {
+                    namespace: Some("my-namespace".to_string()),
+                    cluster_url: Some("https://my-cluster".to_string()),
+                    username: Some("my-username".to_string()),
+                    client_cert: Some("my-cert".to_string()),
+                    client_key: Some("my-key".to_string()),
+                }),
+            },
+        );
+
         let smb_params: Option<SmbParams> = Some(SmbParams {
             share: "test".to_string(),
             workgroup: None,
@@ -498,6 +537,7 @@ mod tests {
                 remote_path: None,
                 local_path: None,
                 s3: None,
+                kube: None,
                 smb: smb_params,
             },
         );
@@ -513,6 +553,7 @@ mod tests {
                 remote_path: Some(PathBuf::from("/tmp")),
                 local_path: Some(PathBuf::from("/usr")),
                 s3: None,
+                kube: None,
                 smb: None,
             },
         );
@@ -548,6 +589,22 @@ mod tests {
         assert!(deserialize::<Theme>(Box::new(toml_file)).is_err());
     }
 
+    #[test]
+    fn test_should_deserialize_v14_pod_bookmark() {
+        let toml = create_v14_pod_bookmark();
+        toml.as_file().sync_all().unwrap();
+        toml.as_file().rewind().unwrap();
+        let deserialized: UserHosts = deserialize(Box::new(toml)).unwrap();
+        let kube = deserialized.bookmarks.get("pod").unwrap();
+        assert_eq!(kube.protocol, FileTransferProtocol::Kube);
+        let kube = kube.kube.as_ref().unwrap();
+        assert_eq!(kube.namespace.as_deref().unwrap(), "my-namespace");
+        assert_eq!(kube.cluster_url.as_deref().unwrap(), "https://my-cluster");
+        assert_eq!(kube.username.as_deref().unwrap(), "my-username");
+        assert_eq!(kube.client_cert.as_deref().unwrap(), "my-cert");
+        assert_eq!(kube.client_key.as_deref().unwrap(), "my-key");
+    }
+
     fn create_good_toml_bookmarks() -> tempfile::NamedTempFile {
         // Write
         let mut tmpfile: tempfile::NamedTempFile = tempfile::NamedTempFile::new().unwrap();
@@ -569,6 +626,15 @@ mod tests {
         secret_access_key = "pluto"
         new_path_style = true
 
+        [bookmarks.pod]
+        protocol = "KUBE"
+        [bookmarks.pod.kube]
+        namespace = "my-namespace"
+        cluster_url = "https://my-cluster"
+        username = "my-username"
+        client_cert = "my-cert"
+        client_key = "my-key"
+
         [bookmarks.smb]
         protocol = "SMB"
         address = "localhost"
@@ -588,6 +654,29 @@ mod tests {
         tmpfile
     }
 
+    fn create_v14_pod_bookmark() -> tempfile::NamedTempFile {
+        let mut tmpfile: tempfile::NamedTempFile = tempfile::NamedTempFile::new().unwrap();
+        let file_content: &str = r#"
+        [bookmarks]
+
+        [bookmarks.pod]
+        protocol = "KUBE"
+        [bookmarks.pod.kube]
+        pod_name = "my-pod"
+        container = "my-container"
+        namespace = "my-namespace"
+        cluster_url = "https://my-cluster"
+        username = "my-username"
+        client_cert = "my-cert"
+        client_key = "my-key"
+
+        [recents]
+        "#;
+        tmpfile.write_all(file_content.as_bytes()).unwrap();
+        //write!(tmpfile, "[bookmarks]\nraspberrypi2 = {{ address = \"192.168.1.31\", port = 22, protocol = \"SFTP\", username = \"root\" }}\nmsi-estrem = {{ address = \"192.168.1.30\", port = 22, protocol = \"SFTP\", username = \"cvisintin\" }}\naws-server-prod1 = {{ address = \"51.23.67.12\", port = 21, protocol = \"FTPS\", username = \"aws001\" }}\n\n[recents]\nISO20201215T094000Z = {{ address = \"172.16.104.10\", port = 22, protocol = \"SCP\", username = \"root\" }}\n");
+        tmpfile
+    }
+
     fn create_bad_toml_bookmarks() -> tempfile::NamedTempFile {
         // Write
         let mut tmpfile: tempfile::NamedTempFile = tempfile::NamedTempFile::new().unwrap();
