@@ -10,7 +10,6 @@ use std::cmp::Reverse;
 use std::collections::VecDeque;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
-use std::string::ToString;
 
 use formatter::Formatter;
 // Ext
@@ -31,6 +30,7 @@ pub enum FileSorting {
     ModifyTime,
     CreationTime,
     Size,
+    None,
 }
 
 /// GroupDirs defines how directories should be grouped in sorting files
@@ -99,13 +99,6 @@ impl FileExplorer {
         }
     }
 
-    /*
-    /// Return amount of files
-    pub fn count(&self) -> usize {
-        self.files.len()
-    }
-    */
-
     /// Iterate over files
     /// Filters are applied based on current options (e.g. hidden files not returned)
     pub fn iter_files(&self) -> impl Iterator<Item = &File> + '_ {
@@ -186,6 +179,7 @@ impl FileExplorer {
             FileSorting::CreationTime => self.sort_files_by_creation_time(),
             FileSorting::ModifyTime => self.sort_files_by_mtime(),
             FileSorting::Size => self.sort_files_by_size(),
+            FileSorting::None => {}
         }
         // Directories first (NOTE: MUST COME AFTER OTHER SORTING)
         // Group directories if necessary
@@ -243,14 +237,19 @@ impl FileExplorer {
 
 // Traits
 
-impl ToString for FileSorting {
-    fn to_string(&self) -> String {
-        String::from(match self {
-            FileSorting::CreationTime => "by_creation_time",
-            FileSorting::ModifyTime => "by_mtime",
-            FileSorting::Name => "by_name",
-            FileSorting::Size => "by_size",
-        })
+impl std::fmt::Display for FileSorting {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "{}",
+            match self {
+                FileSorting::CreationTime => "by_creation_time",
+                FileSorting::ModifyTime => "by_mtime",
+                FileSorting::Name => "by_name",
+                FileSorting::Size => "by_size",
+                FileSorting::None => "none",
+            }
+        )
     }
 }
 
@@ -267,12 +266,16 @@ impl FromStr for FileSorting {
     }
 }
 
-impl ToString for GroupDirs {
-    fn to_string(&self) -> String {
-        String::from(match self {
-            GroupDirs::First => "first",
-            GroupDirs::Last => "last",
-        })
+impl std::fmt::Display for GroupDirs {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "{}",
+            match self {
+                GroupDirs::First => "first",
+                GroupDirs::Last => "last",
+            }
+        )
     }
 }
 
