@@ -2,6 +2,9 @@
 //!
 //! popups components
 
+mod chmod;
+mod goto;
+
 use std::time::UNIX_EPOCH;
 
 use bytesize::ByteSize;
@@ -14,17 +17,15 @@ use tuirealm::props::{
 };
 use tuirealm::{Component, Event, MockComponent, NoUserEvent, State, StateValue};
 #[cfg(unix)]
-use users::{get_group_by_gid, get_user_by_uid};
+use uzers::{get_group_by_gid, get_user_by_uid};
 
+pub use self::chmod::ChmodPopup;
+pub use self::goto::{GotoPopup, ATTR_FILES};
 use super::super::Browser;
 use super::{Msg, PendingActionMsg, TransferMsg, UiMsg};
 use crate::explorer::FileSorting;
 use crate::utils::fmt::fmt_time;
 
-mod chmod;
-
-pub use chmod::ChmodPopup;
-
 #[derive(MockComponent)]
 pub struct CopyPopup {
     component: Input,
@@ -111,6 +112,93 @@ impl Component<Msg, NoUserEvent> for CopyPopup {
     }
 }
 
+#[derive(MockComponent)]
+pub struct FilterPopup {
+    component: Input,
+}
+
+impl FilterPopup {
+    pub fn new(color: Color) -> Self {
+        Self {
+            component: Input::default()
+                .borders(
+                    Borders::default()
+                        .color(color)
+                        .modifiers(BorderType::Rounded),
+                )
+                .foreground(color)
+                .input_type(InputType::Text)
+                .placeholder(
+                    "regex or wildmatch",
+                    Style::default().fg(Color::Rgb(128, 128, 128)),
+                )
+                .title(
+                    "Filter files by regex or wildmatch in the current directory",
+                    Alignment::Center,
+                ),
+        }
+    }
+}
+
+impl Component<Msg, NoUserEvent> for FilterPopup {
+    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+        match ev {
+            Event::Keyboard(KeyEvent {
+                code: Key::Left, ..
+            }) => {
+                self.perform(Cmd::Move(Direction::Left));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Right, ..
+            }) => {
+                self.perform(Cmd::Move(Direction::Right));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Home, ..
+            }) => {
+                self.perform(Cmd::GoTo(Position::Begin));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent { code: Key::End, .. }) => {
+                self.perform(Cmd::GoTo(Position::End));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Delete, ..
+            }) => {
+                self.perform(Cmd::Cancel);
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Backspace,
+                ..
+            }) => {
+                self.perform(Cmd::Delete);
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char(ch),
+                ..
+            }) => {
+                self.perform(Cmd::Type(ch));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Enter, ..
+            }) => match self.state() {
+                State::One(StateValue::String(filter)) => Some(Msg::Ui(UiMsg::FilterFiles(filter))),
+                _ => Some(Msg::None),
+            },
+            Event::Keyboard(KeyEvent { code: Key::Esc, .. }) => {
+                Some(Msg::Ui(UiMsg::CloseFilterPopup))
+            }
+            _ => None,
+        }
+    }
+}
+
 #[derive(MockComponent)]
 pub struct DeletePopup {
     component: Radio,
@@ -496,176 +584,6 @@ impl Component<Msg, NoUserEvent> for FileInfoPopup {
     }
 }
 
-#[derive(MockComponent)]
-pub struct FindPopup {
-    component: Input,
-}
-
-impl FindPopup {
-    pub fn new(color: Color) -> Self {
-        Self {
-            component: Input::default()
-                .borders(
-                    Borders::default()
-                        .color(color)
-                        .modifiers(BorderType::Rounded),
-                )
-                .foreground(color)
-                .input_type(InputType::Text)
-                .placeholder(
-                    "Search files by name",
-                    Style::default().fg(Color::Rgb(128, 128, 128)),
-                )
-                .title("*.txt", Alignment::Center),
-        }
-    }
-}
-
-impl Component<Msg, NoUserEvent> for FindPopup {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
-        match ev {
-            Event::Keyboard(KeyEvent {
-                code: Key::Left, ..
-            }) => {
-                self.perform(Cmd::Move(Direction::Left));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Right, ..
-            }) => {
-                self.perform(Cmd::Move(Direction::Right));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Home, ..
-            }) => {
-                self.perform(Cmd::GoTo(Position::Begin));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent { code: Key::End, .. }) => {
-                self.perform(Cmd::GoTo(Position::End));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Delete, ..
-            }) => {
-                self.perform(Cmd::Cancel);
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Backspace,
-                ..
-            }) => {
-                self.perform(Cmd::Delete);
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Char(ch),
-                ..
-            }) => {
-                self.perform(Cmd::Type(ch));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Enter, ..
-            }) => match self.state() {
-                State::One(StateValue::String(i)) => {
-                    Some(Msg::Transfer(TransferMsg::SearchFile(i)))
-                }
-                _ => Some(Msg::None),
-            },
-            Event::Keyboard(KeyEvent { code: Key::Esc, .. }) => {
-                Some(Msg::Ui(UiMsg::CloseFindPopup))
-            }
-            _ => None,
-        }
-    }
-}
-
-#[derive(MockComponent)]
-pub struct GoToPopup {
-    component: Input,
-}
-
-impl GoToPopup {
-    pub fn new(color: Color) -> Self {
-        Self {
-            component: Input::default()
-                .borders(
-                    Borders::default()
-                        .color(color)
-                        .modifiers(BorderType::Rounded),
-                )
-                .foreground(color)
-                .input_type(InputType::Text)
-                .placeholder(
-                    "/foo/bar/buzz",
-                    Style::default().fg(Color::Rgb(128, 128, 128)),
-                )
-                .title("Go to…", Alignment::Center),
-        }
-    }
-}
-
-impl Component<Msg, NoUserEvent> for GoToPopup {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
-        match ev {
-            Event::Keyboard(KeyEvent {
-                code: Key::Left, ..
-            }) => {
-                self.perform(Cmd::Move(Direction::Left));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Right, ..
-            }) => {
-                self.perform(Cmd::Move(Direction::Right));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Home, ..
-            }) => {
-                self.perform(Cmd::GoTo(Position::Begin));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent { code: Key::End, .. }) => {
-                self.perform(Cmd::GoTo(Position::End));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Delete, ..
-            }) => {
-                self.perform(Cmd::Cancel);
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Backspace,
-                ..
-            }) => {
-                self.perform(Cmd::Delete);
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Char(ch),
-                ..
-            }) => {
-                self.perform(Cmd::Type(ch));
-                Some(Msg::None)
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Enter, ..
-            }) => match self.state() {
-                State::One(StateValue::String(i)) => Some(Msg::Transfer(TransferMsg::GoTo(i))),
-                _ => Some(Msg::None),
-            },
-            Event::Keyboard(KeyEvent { code: Key::Esc, .. }) => {
-                Some(Msg::Ui(UiMsg::CloseGotoPopup))
-            }
-            _ => None,
-        }
-    }
-}
-
 #[derive(MockComponent)]
 pub struct KeybindingsPopup {
     component: List,
@@ -790,12 +708,18 @@ impl KeybindingsPopup {
                         .add_col(TextSpan::new("<Z>").bold().fg(key_color))
                         .add_col(TextSpan::from("               Change file permissions"))
                         .add_row()
+                        .add_col(TextSpan::new("</>").bold().fg(key_color))
+                        .add_col(TextSpan::from("               Filter files"))
+                        .add_row()
                         .add_col(TextSpan::new("<DEL|F8|E>").bold().fg(key_color))
                         .add_col(TextSpan::from("        Delete selected file"))
                         .add_row()
                         .add_col(TextSpan::new("<CTRL+A>").bold().fg(key_color))
                         .add_col(TextSpan::from("          Select all files"))
                         .add_row()
+                        .add_col(TextSpan::new("<ALT+A>").bold().fg(key_color))
+                        .add_col(TextSpan::from("          Deselect all files"))
+                        .add_row()
                         .add_col(TextSpan::new("<CTRL+C>").bold().fg(key_color))
                         .add_col(TextSpan::from("          Interrupt file transfer"))
                         .add_row()
@@ -1112,7 +1036,7 @@ pub struct ProgressBarFull {
 }
 
 impl ProgressBarFull {
-    pub fn new<S: AsRef<str>>(prog: f64, label: S, title: S, color: Color) -> Self {
+    pub fn new<S: Into<String>>(prog: f64, label: S, title: S, color: Color) -> Self {
         Self {
             component: ProgressBar::default()
                 .borders(
@@ -1150,7 +1074,7 @@ pub struct ProgressBarPartial {
 }
 
 impl ProgressBarPartial {
-    pub fn new<S: AsRef<str>>(prog: f64, label: S, title: S, color: Color) -> Self {
+    pub fn new<S: Into<String>>(prog: f64, label: S, title: S, color: Color) -> Self {
         Self {
             component: ProgressBar::default()
                 .borders(
@@ -1585,6 +1509,7 @@ impl SortingPopup {
                     FileSorting::ModifyTime => 1,
                     FileSorting::Name => 0,
                     FileSorting::Size => 3,
+                    FileSorting::None => 0,
                 }),
         }
     }
@@ -1626,8 +1551,8 @@ pub struct StatusBarLocal {
 
 impl StatusBarLocal {
     pub fn new(browser: &Browser, sorting_color: Color, hidden_color: Color) -> Self {
-        let file_sorting = file_sorting_label(browser.local().file_sorting);
-        let hidden_files = hidden_files_label(browser.local().hidden_files_visible());
+        let file_sorting = file_sorting_label(browser.host_bridge().file_sorting);
+        let hidden_files = hidden_files_label(browser.host_bridge().hidden_files_visible());
         Self {
             component: Span::default().spans(&[
                 TextSpan::new("File sorting: ").fg(sorting_color),
@@ -1688,6 +1613,7 @@ fn file_sorting_label(sorting: FileSorting) -> &'static str {
         FileSorting::ModifyTime => "By modify time",
         FileSorting::Name => "By name",
         FileSorting::Size => "By size",
+        FileSorting::None => "",
     }
 }
 
@@ -1888,6 +1814,47 @@ impl Component<Msg, NoUserEvent> for WaitPopup {
     }
 }
 
+#[derive(MockComponent)]
+pub struct WalkdirWaitPopup {
+    component: Paragraph,
+}
+
+impl WalkdirWaitPopup {
+    pub fn new<S: AsRef<str>>(text: S, color: Color) -> Self {
+        Self {
+            component: Paragraph::default()
+                .alignment(Alignment::Center)
+                .borders(
+                    Borders::default()
+                        .color(color)
+                        .modifiers(BorderType::Rounded),
+                )
+                .foreground(color)
+                .text(&[
+                    TextSpan::from(text.as_ref()),
+                    TextSpan::from("Press 'CTRL+C' to abort"),
+                ])
+                .wrap(true),
+        }
+    }
+}
+
+impl Component<Msg, NoUserEvent> for WalkdirWaitPopup {
+    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+        if matches!(
+            ev,
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('c'),
+                modifiers: KeyModifiers::CONTROL
+            })
+        ) {
+            Some(Msg::Transfer(TransferMsg::AbortWalkdir))
+        } else {
+            None
+        }
+    }
+}
+
 #[derive(MockComponent)]
 pub struct WatchedPathsList {
     component: List,
