@@ -2,14 +2,220 @@
 //!
 //! file transfer components
 
-use super::{Msg, TransferMsg, UiMsg};
-
 mod file_list;
-use file_list::FileList;
-use tuirealm::command::{Cmd, Direction, Position};
+mod file_list_with_search;
+
+use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers};
 use tuirealm::props::{Alignment, Borders, Color, TextSpan};
-use tuirealm::{Component, Event, MockComponent, NoUserEvent};
+use tuirealm::{Component, Event, MockComponent, NoUserEvent, State, StateValue};
+
+use self::file_list::FileList;
+use self::file_list_with_search::FileListWithSearch;
+use super::{Msg, TransferMsg, UiMsg};
+
+#[derive(MockComponent)]
+pub struct ExplorerFuzzy {
+    component: FileListWithSearch,
+}
+
+impl ExplorerFuzzy {
+    pub fn new<S: AsRef<str>>(title: S, files: &[&str], bg: Color, fg: Color, hg: Color) -> Self {
+        Self {
+            component: FileListWithSearch::default()
+                .background(bg)
+                .borders(Borders::default().color(hg))
+                .foreground(fg)
+                .highlighted_color(hg)
+                .title(title, Alignment::Left)
+                .rows(files.iter().map(|x| vec![TextSpan::from(x)]).collect()),
+        }
+    }
+
+    fn on_search(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+        match ev {
+            Event::Keyboard(KeyEvent {
+                code: Key::Left, ..
+            }) => {
+                self.perform(Cmd::Move(Direction::Left));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Right, ..
+            }) => {
+                self.perform(Cmd::Move(Direction::Right));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Home, ..
+            }) => {
+                self.perform(Cmd::GoTo(Position::Begin));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent { code: Key::End, .. }) => {
+                self.perform(Cmd::GoTo(Position::End));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Delete, ..
+            }) => {
+                self.perform(Cmd::Cancel);
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Backspace,
+                ..
+            }) => {
+                self.perform(Cmd::Delete);
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Tab | Key::Up | Key::Down,
+                ..
+            }) => {
+                self.perform(Cmd::Change);
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char(ch),
+                ..
+            }) => match self.perform(Cmd::Type(ch)) {
+                CmdResult::Changed(State::One(StateValue::String(search))) => {
+                    Some(Msg::Ui(UiMsg::FuzzySearch(search)))
+                }
+                _ => Some(Msg::None),
+            },
+            Event::Keyboard(KeyEvent { code: Key::Esc, .. }) => {
+                Some(Msg::Ui(UiMsg::CloseFindExplorer))
+            }
+            _ => None,
+        }
+    }
+
+    fn on_file_list(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+        match ev {
+            Event::Keyboard(KeyEvent {
+                code: Key::Down, ..
+            }) => {
+                self.perform(Cmd::Move(Direction::Down));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent { code: Key::Up, .. }) => {
+                self.perform(Cmd::Move(Direction::Up));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::PageDown,
+                ..
+            }) => {
+                self.perform(Cmd::Scroll(Direction::Down));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::PageUp, ..
+            }) => {
+                self.perform(Cmd::Scroll(Direction::Up));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Home, ..
+            }) => {
+                self.perform(Cmd::GoTo(Position::Begin));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent { code: Key::End, .. }) => {
+                self.perform(Cmd::GoTo(Position::End));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('a'),
+                modifiers: KeyModifiers::CONTROL,
+            }) => {
+                let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_SELECT_ALL));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('a'),
+                modifiers: KeyModifiers::ALT,
+            }) => {
+                let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_DESELECT_ALL));
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('m'),
+                modifiers: KeyModifiers::NONE,
+            }) => {
+                let _ = self.perform(Cmd::Toggle);
+                Some(Msg::None)
+            }
+            Event::Keyboard(KeyEvent { code: Key::Tab, .. }) => {
+                self.perform(Cmd::Change);
+                Some(Msg::None)
+            }
+            // -- comp msg
+            Event::Keyboard(KeyEvent { code: Key::Esc, .. }) => {
+                Some(Msg::Ui(UiMsg::CloseFindExplorer))
+            }
+            Event::Keyboard(KeyEvent {
+                code: Key::Left | Key::Right,
+                ..
+            }) => Some(Msg::Ui(UiMsg::ChangeTransferWindow)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Enter, ..
+            }) => Some(Msg::Transfer(TransferMsg::EnterDirectory)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char(' '),
+                ..
+            }) => Some(Msg::Transfer(TransferMsg::TransferFile)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Backspace,
+                ..
+            }) => Some(Msg::Transfer(TransferMsg::GoToPreviousDirectory)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('a'),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ToggleHiddenFiles)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('b'),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowFileSortingPopup)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('e') | Key::Delete | Key::Function(8),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowDeletePopup)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('i'),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowFileInfoPopup)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('s') | Key::Function(2),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowSaveAsPopup)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('v') | Key::Function(3),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Transfer(TransferMsg::OpenFile)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('w'),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowOpenWithPopup)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('z'),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowChmodPopup)),
+            _ => None,
+        }
+    }
+}
+
+impl Component<Msg, NoUserEvent> for ExplorerFuzzy {
+    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+        match self.component.focus() {
+            file_list_with_search::Focus::List => self.on_file_list(ev),
+            file_list_with_search::Focus::Search => self.on_search(ev),
+        }
+    }
+}
 
 #[derive(MockComponent)]
 pub struct ExplorerFind {
@@ -73,6 +279,13 @@ impl Component<Msg, NoUserEvent> for ExplorerFind {
                 let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_SELECT_ALL));
                 Some(Msg::None)
             }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('a'),
+                modifiers: KeyModifiers::ALT,
+            }) => {
+                let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_DESELECT_ALL));
+                Some(Msg::None)
+            }
             Event::Keyboard(KeyEvent {
                 code: Key::Char('m'),
                 modifiers: KeyModifiers::NONE,
@@ -150,7 +363,8 @@ impl ExplorerLocal {
                 .foreground(fg)
                 .highlighted_color(hg)
                 .title(title, Alignment::Left)
-                .rows(files.iter().map(|x| vec![TextSpan::from(x)]).collect()),
+                .rows(files.iter().map(|x| vec![TextSpan::from(x)]).collect())
+                .dot_dot(true),
         }
     }
 }
@@ -198,6 +412,13 @@ impl Component<Msg, NoUserEvent> for ExplorerLocal {
                 let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_SELECT_ALL));
                 Some(Msg::None)
             }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('a'),
+                modifiers: KeyModifiers::ALT,
+            }) => {
+                let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_DESELECT_ALL));
+                Some(Msg::None)
+            }
             Event::Keyboard(KeyEvent {
                 code: Key::Char('m'),
                 modifiers: KeyModifiers::NONE,
@@ -219,11 +440,23 @@ impl Component<Msg, NoUserEvent> for ExplorerLocal {
             }) => Some(Msg::Transfer(TransferMsg::GoToPreviousDirectory)),
             Event::Keyboard(KeyEvent {
                 code: Key::Enter, ..
-            }) => Some(Msg::Transfer(TransferMsg::EnterDirectory)),
+            }) => {
+                if matches!(self.component.state(), State::One(StateValue::String(_))) {
+                    Some(Msg::Transfer(TransferMsg::GoToParentDirectory))
+                } else {
+                    Some(Msg::Transfer(TransferMsg::EnterDirectory))
+                }
+            }
             Event::Keyboard(KeyEvent {
                 code: Key::Char(' '),
                 ..
-            }) => Some(Msg::Transfer(TransferMsg::TransferFile)),
+            }) => {
+                if matches!(self.component.state(), State::One(StateValue::String(_))) {
+                    Some(Msg::None)
+                } else {
+                    Some(Msg::Transfer(TransferMsg::TransferFile))
+                }
+            }
             Event::Keyboard(KeyEvent {
                 code: Key::Char('a'),
                 modifiers: KeyModifiers::NONE,
@@ -247,7 +480,7 @@ impl Component<Msg, NoUserEvent> for ExplorerLocal {
             Event::Keyboard(KeyEvent {
                 code: Key::Char('f'),
                 modifiers: KeyModifiers::NONE,
-            }) => Some(Msg::Ui(UiMsg::ShowFindPopup)),
+            }) => Some(Msg::Transfer(TransferMsg::InitFuzzySearch)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('g'),
                 modifiers: KeyModifiers::NONE,
@@ -316,6 +549,10 @@ impl Component<Msg, NoUserEvent> for ExplorerLocal {
                 code: Key::Char('z'),
                 modifiers: KeyModifiers::NONE,
             }) => Some(Msg::Ui(UiMsg::ShowChmodPopup)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('/'),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowFilterPopup)),
             _ => None,
         }
     }
@@ -335,7 +572,8 @@ impl ExplorerRemote {
                 .foreground(fg)
                 .highlighted_color(hg)
                 .title(title, Alignment::Left)
-                .rows(files.iter().map(|x| vec![TextSpan::from(x)]).collect()),
+                .rows(files.iter().map(|x| vec![TextSpan::from(x)]).collect())
+                .dot_dot(true),
         }
     }
 }
@@ -383,6 +621,13 @@ impl Component<Msg, NoUserEvent> for ExplorerRemote {
                 let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_SELECT_ALL));
                 Some(Msg::None)
             }
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('a'),
+                modifiers: KeyModifiers::ALT,
+            }) => {
+                let _ = self.perform(Cmd::Custom(file_list::FILE_LIST_CMD_DESELECT_ALL));
+                Some(Msg::None)
+            }
             Event::Keyboard(KeyEvent {
                 code: Key::Char('m'),
                 modifiers: KeyModifiers::NONE,
@@ -404,11 +649,23 @@ impl Component<Msg, NoUserEvent> for ExplorerRemote {
             }) => Some(Msg::Transfer(TransferMsg::GoToPreviousDirectory)),
             Event::Keyboard(KeyEvent {
                 code: Key::Enter, ..
-            }) => Some(Msg::Transfer(TransferMsg::EnterDirectory)),
+            }) => {
+                if matches!(self.component.state(), State::One(StateValue::String(_))) {
+                    Some(Msg::Transfer(TransferMsg::GoToParentDirectory))
+                } else {
+                    Some(Msg::Transfer(TransferMsg::EnterDirectory))
+                }
+            }
             Event::Keyboard(KeyEvent {
                 code: Key::Char(' '),
                 ..
-            }) => Some(Msg::Transfer(TransferMsg::TransferFile)),
+            }) => {
+                if matches!(self.component.state(), State::One(StateValue::String(_))) {
+                    Some(Msg::None)
+                } else {
+                    Some(Msg::Transfer(TransferMsg::TransferFile))
+                }
+            }
             Event::Keyboard(KeyEvent {
                 code: Key::Char('a'),
                 modifiers: KeyModifiers::NONE,
@@ -432,7 +689,7 @@ impl Component<Msg, NoUserEvent> for ExplorerRemote {
             Event::Keyboard(KeyEvent {
                 code: Key::Char('f'),
                 modifiers: KeyModifiers::NONE,
-            }) => Some(Msg::Ui(UiMsg::ShowFindPopup)),
+            }) => Some(Msg::Transfer(TransferMsg::InitFuzzySearch)),
             Event::Keyboard(KeyEvent {
                 code: Key::Char('g'),
                 modifiers: KeyModifiers::NONE,
@@ -501,6 +758,10 @@ impl Component<Msg, NoUserEvent> for ExplorerRemote {
                 code: Key::Char('z'),
                 modifiers: KeyModifiers::NONE,
             }) => Some(Msg::Ui(UiMsg::ShowChmodPopup)),
+            Event::Keyboard(KeyEvent {
+                code: Key::Char('/'),
+                modifiers: KeyModifiers::NONE,
+            }) => Some(Msg::Ui(UiMsg::ShowFilterPopup)),
             _ => None,
         }
     }
