@@ -8,6 +8,7 @@ use remotefs::fs::File;
 use tuirealm::props::{AttrValue, Attribute};
 use tuirealm::{State, StateValue, Update};
 
+use super::actions::walkdir::WalkdirError;
 use super::actions::SelectedFile;
 use super::browser::{FileExplorerTab, FoundExplorerTab};
 use super::{ExitReason, FileTransferActivity, Id, Msg, TransferMsg, TransferOpts, UiMsg};
@@ -32,18 +33,19 @@ impl FileTransferActivity {
             TransferMsg::AbortTransfer => {
                 self.transfer.abort();
             }
+            TransferMsg::AbortWalkdir => {
+                self.walkdir.aborted = true;
+            }
             TransferMsg::Chmod(mode) => {
                 self.umount_chmod();
                 self.mount_blocking_wait("Applying new file mode…");
                 match self.browser.tab() {
-                    #[cfg(unix)]
-                    FileExplorerTab::Local => self.action_local_chmod(mode),
-                    #[cfg(unix)]
-                    FileExplorerTab::FindLocal => self.action_find_local_chmod(mode),
+                    FileExplorerTab::HostBridge | FileExplorerTab::FindHostBridge
+                        if self.host_bridge.is_localhost() && cfg!(windows) => {}
+                    FileExplorerTab::HostBridge => self.action_local_chmod(mode),
+                    FileExplorerTab::FindHostBridge => self.action_find_local_chmod(mode),
                     FileExplorerTab::Remote => self.action_remote_chmod(mode),
                     FileExplorerTab::FindRemote => self.action_find_remote_chmod(mode),
-                    #[cfg(windows)]
-                    FileExplorerTab::Local | FileExplorerTab::FindLocal => {}
                 }
                 self.umount_wait();
                 self.update_browser_file_list();
@@ -52,7 +54,7 @@ impl FileTransferActivity {
                 self.umount_copy();
                 self.mount_blocking_wait("Copying file(s)…");
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_copy(dest),
+                    FileExplorerTab::HostBridge => self.action_local_copy(dest),
                     FileExplorerTab::Remote => self.action_remote_copy(dest),
                     _ => panic!("Found tab doesn't support COPY"),
                 }
@@ -64,7 +66,7 @@ impl FileTransferActivity {
                 self.umount_symlink();
                 self.mount_blocking_wait("Creating symlink…");
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_symlink(name),
+                    FileExplorerTab::HostBridge => self.action_local_symlink(name),
                     FileExplorerTab::Remote => self.action_remote_symlink(name),
                     _ => panic!("Found tab doesn't support SYMLINK"),
                 }
@@ -76,9 +78,9 @@ impl FileTransferActivity {
                 self.umount_radio_delete();
                 self.mount_blocking_wait("Removing file(s)…");
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_delete(),
+                    FileExplorerTab::HostBridge => self.action_local_delete(),
                     FileExplorerTab::Remote => self.action_remote_delete(),
-                    FileExplorerTab::FindLocal | FileExplorerTab::FindRemote => {
+                    FileExplorerTab::FindHostBridge | FileExplorerTab::FindRemote => {
                         // Get entry
                         self.action_find_delete();
                         // Delete entries
@@ -104,20 +106,20 @@ impl FileTransferActivity {
                 self.umount_wait();
                 // Reload files
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.update_local_filelist(),
+                    FileExplorerTab::HostBridge => self.update_host_bridge_filelist(),
                     FileExplorerTab::Remote => self.update_remote_filelist(),
-                    FileExplorerTab::FindLocal => self.update_local_filelist(),
+                    FileExplorerTab::FindHostBridge => self.update_host_bridge_filelist(),
                     FileExplorerTab::FindRemote => self.update_remote_filelist(),
                 }
             }
-            TransferMsg::EnterDirectory if self.browser.tab() == FileExplorerTab::Local => {
+            TransferMsg::EnterDirectory if self.browser.tab() == FileExplorerTab::HostBridge => {
                 if let SelectedFile::One(entry) = self.get_local_selected_entries() {
                     self.action_submit_local(entry);
                     // Update file list if sync
                     if self.browser.sync_browsing && self.browser.found().is_none() {
                         self.update_remote_filelist();
                     }
-                    self.update_local_filelist();
+                    self.update_host_bridge_filelist();
                 }
             }
             TransferMsg::EnterDirectory if self.browser.tab() == FileExplorerTab::Remote => {
@@ -125,7 +127,7 @@ impl FileTransferActivity {
                     self.action_submit_remote(entry);
                     // Update file list if sync
                     if self.browser.sync_browsing && self.browser.found().is_none() {
-                        self.update_local_filelist();
+                        self.update_host_bridge_filelist();
                     }
                     self.update_remote_filelist();
                 }
@@ -146,7 +148,7 @@ impl FileTransferActivity {
                 self.umount_exec();
                 self.mount_blocking_wait(format!("Executing '{cmd}'…").as_str());
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_exec(cmd),
+                    FileExplorerTab::HostBridge => self.action_local_exec(cmd),
                     FileExplorerTab::Remote => self.action_remote_exec(cmd),
                     _ => panic!("Found tab doesn't support EXEC"),
                 }
@@ -156,7 +158,7 @@ impl FileTransferActivity {
             }
             TransferMsg::GoTo(dir) => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_change_local_dir(dir),
+                    FileExplorerTab::HostBridge => self.action_change_local_dir(dir),
                     FileExplorerTab::Remote => self.action_change_remote_dir(dir),
                     _ => panic!("Found tab doesn't support GOTO"),
                 }
@@ -171,18 +173,18 @@ impl FileTransferActivity {
             }
             TransferMsg::GoToParentDirectory => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => {
+                    FileExplorerTab::HostBridge => {
                         self.action_go_to_local_upper_dir();
                         if self.browser.sync_browsing && self.browser.found().is_none() {
                             self.update_remote_filelist();
                         }
                         // Reload file list component
-                        self.update_local_filelist()
+                        self.update_host_bridge_filelist()
                     }
                     FileExplorerTab::Remote => {
                         self.action_go_to_remote_upper_dir();
                         if self.browser.sync_browsing && self.browser.found().is_none() {
-                            self.update_local_filelist();
+                            self.update_host_bridge_filelist();
                         }
                         // Reload file list component
                         self.update_remote_filelist()
@@ -192,18 +194,18 @@ impl FileTransferActivity {
             }
             TransferMsg::GoToPreviousDirectory => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => {
+                    FileExplorerTab::HostBridge => {
                         self.action_go_to_previous_local_dir();
                         if self.browser.sync_browsing && self.browser.found().is_none() {
                             self.update_remote_filelist();
                         }
                         // Reload file list component
-                        self.update_local_filelist()
+                        self.update_host_bridge_filelist()
                     }
                     FileExplorerTab::Remote => {
                         self.action_go_to_previous_remote_dir();
                         if self.browser.sync_browsing && self.browser.found().is_none() {
-                            self.update_local_filelist();
+                            self.update_host_bridge_filelist();
                         }
                         // Reload file list component
                         self.update_remote_filelist()
@@ -211,9 +213,62 @@ impl FileTransferActivity {
                     _ => {}
                 }
             }
+            TransferMsg::InitFuzzySearch => {
+                // Mount wait
+                self.mount_walkdir_wait();
+                // Find
+                let res: Result<Vec<File>, WalkdirError> = match self.browser.tab() {
+                    FileExplorerTab::HostBridge => self.action_walkdir_local(),
+                    FileExplorerTab::Remote => self.action_walkdir_remote(),
+                    _ => panic!("Trying to search for files, while already in a find result"),
+                };
+                // Umount wait
+                self.umount_wait();
+                // Match result
+                match res {
+                    Err(WalkdirError::Error(err)) => {
+                        // Mount error
+                        self.mount_error(err.as_str());
+                    }
+                    Err(WalkdirError::Aborted) => {
+                        self.mount_info("Search aborted");
+                    }
+                    Ok(files) if files.is_empty() => {
+                        // If no file has been found notify user
+                        self.mount_info("There are no files in the current directory");
+                    }
+                    Ok(files) => {
+                        // Get wrkdir
+                        let wrkdir = match self.browser.tab() {
+                            FileExplorerTab::HostBridge => self.host_bridge().wrkdir.clone(),
+                            _ => self.remote().wrkdir.clone(),
+                        };
+                        // Create explorer and load files
+                        self.browser.set_found(
+                            match self.browser.tab() {
+                                FileExplorerTab::HostBridge => FoundExplorerTab::Local,
+                                _ => FoundExplorerTab::Remote,
+                            },
+                            files,
+                            wrkdir.as_path(),
+                        );
+                        // init fuzzy search to display nothing
+                        self.browser.init_fuzzy_search();
+                        // Mount result widget
+                        self.mount_find(format!(r#"Searching at "{}""#, wrkdir.display()), true);
+                        self.update_find_list();
+                        // Initialize tab
+                        self.browser.change_tab(match self.browser.tab() {
+                            FileExplorerTab::HostBridge => FileExplorerTab::FindHostBridge,
+                            FileExplorerTab::Remote => FileExplorerTab::FindRemote,
+                            _ => FileExplorerTab::FindHostBridge,
+                        });
+                    }
+                }
+            }
             TransferMsg::Mkdir(dir) => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_mkdir(dir),
+                    FileExplorerTab::HostBridge => self.action_local_mkdir(dir),
                     FileExplorerTab::Remote => self.action_remote_mkdir(dir),
                     _ => {}
                 }
@@ -223,7 +278,7 @@ impl FileTransferActivity {
             }
             TransferMsg::NewFile(name) => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_newfile(name),
+                    FileExplorerTab::HostBridge => self.action_local_newfile(name),
                     FileExplorerTab::Remote => self.action_remote_newfile(name),
                     _ => {}
                 }
@@ -232,15 +287,17 @@ impl FileTransferActivity {
                 self.update_browser_file_list()
             }
             TransferMsg::OpenFile => match self.browser.tab() {
-                FileExplorerTab::Local => self.action_open_local(),
+                FileExplorerTab::HostBridge => self.action_open_local(),
                 FileExplorerTab::Remote => self.action_open_remote(),
-                FileExplorerTab::FindLocal | FileExplorerTab::FindRemote => self.action_find_open(),
+                FileExplorerTab::FindHostBridge | FileExplorerTab::FindRemote => {
+                    self.action_find_open()
+                }
             },
             TransferMsg::OpenFileWith(prog) => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_open_with(&prog),
+                    FileExplorerTab::HostBridge => self.action_local_open_with(&prog),
                     FileExplorerTab::Remote => self.action_remote_open_with(&prog),
-                    FileExplorerTab::FindLocal | FileExplorerTab::FindRemote => {
+                    FileExplorerTab::FindHostBridge | FileExplorerTab::FindRemote => {
                         self.action_find_open_with(&prog)
                     }
                 }
@@ -248,7 +305,7 @@ impl FileTransferActivity {
             }
             TransferMsg::OpenTextFile => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_edit_local_file(),
+                    FileExplorerTab::HostBridge => self.action_edit_local_file(),
                     FileExplorerTab::Remote => self.action_edit_remote_file(),
                     _ => {}
                 }
@@ -259,7 +316,7 @@ impl FileTransferActivity {
                 self.umount_rename();
                 self.mount_blocking_wait("Moving file(s)…");
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_rename(dest),
+                    FileExplorerTab::HostBridge => self.action_local_rename(dest),
                     FileExplorerTab::Remote => self.action_remote_rename(dest),
                     _ => {}
                 }
@@ -267,12 +324,21 @@ impl FileTransferActivity {
                 // Reload files
                 self.update_browser_file_list()
             }
+            TransferMsg::RescanGotoFiles(path) => {
+                let files = self.action_scan(&path).unwrap_or_default();
+                let files = files
+                    .into_iter()
+                    .filter(|f| f.is_dir() || f.is_symlink())
+                    .map(|f| f.path().to_string_lossy().to_string())
+                    .collect();
+                self.update_goto(files);
+            }
             TransferMsg::SaveFileAs(dest) => {
                 self.umount_saveas();
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_saveas(dest),
+                    FileExplorerTab::HostBridge => self.action_local_saveas(dest),
                     FileExplorerTab::Remote => self.action_remote_saveas(dest),
-                    FileExplorerTab::FindLocal | FileExplorerTab::FindRemote => {
+                    FileExplorerTab::FindHostBridge | FileExplorerTab::FindRemote => {
                         // Get entry
                         self.action_find_transfer(TransferOpts::default().save_as(Some(dest)));
                     }
@@ -281,64 +347,14 @@ impl FileTransferActivity {
                 // Reload files
                 self.update_browser_file_list_swapped();
             }
-            TransferMsg::SearchFile(search) => {
-                self.umount_find_input();
-                // Mount wait
-                self.mount_blocking_wait(format!(r#"Searching for "{search}"…"#).as_str());
-                // Find
-                let res: Result<Vec<File>, String> = match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_find(search.clone()),
-                    FileExplorerTab::Remote => self.action_remote_find(search.clone()),
-                    _ => panic!("Trying to search for files, while already in a find result"),
-                };
-                // Umount wait
-                self.umount_wait();
-                // Match result
-                match res {
-                    Err(err) => {
-                        // Mount error
-                        self.mount_error(err.as_str());
-                    }
-                    Ok(files) if files.is_empty() => {
-                        // If no file has been found notify user
-                        self.mount_info(
-                            format!(r#"Could not find any file matching "{search}""#).as_str(),
-                        );
-                    }
-                    Ok(files) => {
-                        // Get wrkdir
-                        let wrkdir = match self.browser.tab() {
-                            FileExplorerTab::Local => self.local().wrkdir.clone(),
-                            _ => self.remote().wrkdir.clone(),
-                        };
-                        // Create explorer and load files
-                        self.browser.set_found(
-                            match self.browser.tab() {
-                                FileExplorerTab::Local => FoundExplorerTab::Local,
-                                _ => FoundExplorerTab::Remote,
-                            },
-                            files,
-                            wrkdir.as_path(),
-                        );
-                        // Mount result widget
-                        self.mount_find(&search);
-                        self.update_find_list();
-                        // Initialize tab
-                        self.browser.change_tab(match self.browser.tab() {
-                            FileExplorerTab::Local => FileExplorerTab::FindLocal,
-                            FileExplorerTab::Remote => FileExplorerTab::FindRemote,
-                            _ => FileExplorerTab::FindLocal,
-                        });
-                    }
-                }
-            }
+
             TransferMsg::ToggleWatch => self.action_toggle_watch(),
             TransferMsg::ToggleWatchFor(index) => self.action_toggle_watch_for(index),
             TransferMsg::TransferFile => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local => self.action_local_send(),
+                    FileExplorerTab::HostBridge => self.action_local_send(),
                     FileExplorerTab::Remote => self.action_remote_recv(),
-                    FileExplorerTab::FindLocal | FileExplorerTab::FindRemote => {
+                    FileExplorerTab::FindHostBridge | FileExplorerTab::FindRemote => {
                         self.action_find_transfer(TransferOpts::default())
                     }
                 }
@@ -355,8 +371,8 @@ impl FileTransferActivity {
             UiMsg::CloseChmodPopup => self.umount_chmod(),
             UiMsg::ChangeFileSorting(sorting) => {
                 match self.browser.tab() {
-                    FileExplorerTab::Local | FileExplorerTab::FindLocal => {
-                        self.local_mut().sort_by(sorting);
+                    FileExplorerTab::HostBridge | FileExplorerTab::FindHostBridge => {
+                        self.host_bridge_mut().sort_by(sorting);
                         self.refresh_local_status_bar();
                     }
                     FileExplorerTab::Remote | FileExplorerTab::FindRemote => {
@@ -368,22 +384,28 @@ impl FileTransferActivity {
             }
             UiMsg::ChangeTransferWindow => {
                 let new_tab = match self.browser.tab() {
-                    FileExplorerTab::Local if self.browser.found().is_some() => {
+                    FileExplorerTab::HostBridge if self.browser.found().is_some() => {
                         FileExplorerTab::FindRemote
                     }
-                    FileExplorerTab::FindLocal | FileExplorerTab::Local => FileExplorerTab::Remote,
+                    FileExplorerTab::FindHostBridge | FileExplorerTab::HostBridge => {
+                        FileExplorerTab::Remote
+                    }
                     FileExplorerTab::Remote if self.browser.found().is_some() => {
-                        FileExplorerTab::FindLocal
+                        FileExplorerTab::FindHostBridge
+                    }
+                    FileExplorerTab::FindRemote | FileExplorerTab::Remote => {
+                        FileExplorerTab::HostBridge
                     }
-                    FileExplorerTab::FindRemote | FileExplorerTab::Remote => FileExplorerTab::Local,
                 };
                 // Set focus
                 match new_tab {
-                    FileExplorerTab::Local => assert!(self.app.active(&Id::ExplorerLocal).is_ok()),
+                    FileExplorerTab::HostBridge => {
+                        assert!(self.app.active(&Id::ExplorerHostBridge).is_ok())
+                    }
                     FileExplorerTab::Remote => {
                         assert!(self.app.active(&Id::ExplorerRemote).is_ok())
                     }
-                    FileExplorerTab::FindLocal | FileExplorerTab::FindRemote => {
+                    FileExplorerTab::FindHostBridge | FileExplorerTab::FindRemote => {
                         assert!(self.app.active(&Id::ExplorerFind).is_ok())
                     }
                 }
@@ -400,11 +422,11 @@ impl FileTransferActivity {
             }
             UiMsg::CloseFileInfoPopup => self.umount_file_info(),
             UiMsg::CloseFileSortingPopup => self.umount_file_sorting(),
+            UiMsg::CloseFilterPopup => self.umount_filter(),
             UiMsg::CloseFindExplorer => {
                 self.finalize_find();
                 self.umount_find();
             }
-            UiMsg::CloseFindPopup => self.umount_find_input(),
             UiMsg::CloseGotoPopup => self.umount_goto(),
             UiMsg::CloseKeybindingsPopup => self.umount_help(),
             UiMsg::CloseMkdirPopup => self.umount_mkdir(),
@@ -420,11 +442,42 @@ impl FileTransferActivity {
                 self.disconnect();
                 self.umount_disconnect();
             }
+            UiMsg::FilterFiles(filter) => {
+                self.umount_filter();
+                let files = self.filter(&filter);
+                // Get wrkdir
+                let wrkdir = match self.browser.tab() {
+                    FileExplorerTab::HostBridge => self.host_bridge().wrkdir.clone(),
+                    _ => self.remote().wrkdir.clone(),
+                };
+                // Create explorer and load files
+                self.browser.set_found(
+                    match self.browser.tab() {
+                        FileExplorerTab::HostBridge => FoundExplorerTab::Local,
+                        _ => FoundExplorerTab::Remote,
+                    },
+                    files,
+                    wrkdir.as_path(),
+                );
+                // Mount result widget
+                self.mount_find(&filter, false);
+                self.update_find_list();
+                // Initialize tab
+                self.browser.change_tab(match self.browser.tab() {
+                    FileExplorerTab::HostBridge => FileExplorerTab::FindHostBridge,
+                    FileExplorerTab::Remote => FileExplorerTab::FindRemote,
+                    _ => FileExplorerTab::FindHostBridge,
+                });
+            }
+            UiMsg::FuzzySearch(needle) => {
+                self.browser.fuzzy_search(&needle);
+                self.update_find_list();
+            }
             UiMsg::ShowLogPanel => {
                 assert!(self.app.active(&Id::Log).is_ok());
             }
             UiMsg::LogBackTabbed => {
-                assert!(self.app.active(&Id::ExplorerLocal).is_ok());
+                assert!(self.app.active(&Id::ExplorerHostBridge).is_ok());
             }
             UiMsg::Quit => {
                 self.disconnect_and_quit();
@@ -442,13 +495,15 @@ impl FileTransferActivity {
             UiMsg::ShowChmodPopup => {
                 let selected_file = match self.browser.tab() {
                     #[cfg(unix)]
-                    FileExplorerTab::Local => self.get_local_selected_entries(),
+                    FileExplorerTab::HostBridge => self.get_local_selected_entries(),
                     #[cfg(unix)]
-                    FileExplorerTab::FindLocal => self.get_found_selected_entries(),
+                    FileExplorerTab::FindHostBridge => self.get_found_selected_entries(),
                     FileExplorerTab::Remote => self.get_remote_selected_entries(),
                     FileExplorerTab::FindRemote => self.get_found_selected_entries(),
                     #[cfg(windows)]
-                    FileExplorerTab::Local | FileExplorerTab::FindLocal => SelectedFile::None,
+                    FileExplorerTab::HostBridge | FileExplorerTab::FindHostBridge => {
+                        SelectedFile::None
+                    }
                 };
                 if let Some(mode) = selected_file.unix_pex() {
                     self.mount_chmod(
@@ -469,7 +524,7 @@ impl FileTransferActivity {
             UiMsg::ShowDeletePopup => self.mount_radio_delete(),
             UiMsg::ShowDisconnectPopup => self.mount_disconnect(),
             UiMsg::ShowExecPopup => self.mount_exec(),
-            UiMsg::ShowFileInfoPopup if self.browser.tab() == FileExplorerTab::Local => {
+            UiMsg::ShowFileInfoPopup if self.browser.tab() == FileExplorerTab::HostBridge => {
                 if let SelectedFile::One(file) = self.get_local_selected_entries() {
                     self.mount_file_info(&file);
                 }
@@ -485,7 +540,7 @@ impl FileTransferActivity {
                 }
             }
             UiMsg::ShowFileSortingPopup => self.mount_file_sorting(),
-            UiMsg::ShowFindPopup => self.mount_find_input(),
+            UiMsg::ShowFilterPopup => self.mount_filter(),
             UiMsg::ShowGotoPopup => self.mount_goto(),
             UiMsg::ShowKeybindingsPopup => self.mount_help(),
             UiMsg::ShowMkdirPopup => self.mount_mkdir(),
@@ -496,9 +551,9 @@ impl FileTransferActivity {
             UiMsg::ShowSaveAsPopup => self.mount_saveas(),
             UiMsg::ShowSymlinkPopup => {
                 if match self.browser.tab() {
-                    FileExplorerTab::Local => self.is_local_selected_one(),
+                    FileExplorerTab::HostBridge => self.is_local_selected_one(),
                     FileExplorerTab::Remote => self.is_remote_selected_one(),
-                    FileExplorerTab::FindLocal | FileExplorerTab::FindRemote => false,
+                    FileExplorerTab::FindHostBridge | FileExplorerTab::FindRemote => false,
                 } {
                     // Only if only one entry is selected
                     self.mount_symlink();
@@ -511,8 +566,8 @@ impl FileTransferActivity {
             UiMsg::ShowWatchedPathsList => self.action_show_watched_paths_list(),
             UiMsg::ShowWatcherPopup => self.action_show_radio_watch(),
             UiMsg::ToggleHiddenFiles => match self.browser.tab() {
-                FileExplorerTab::FindLocal | FileExplorerTab::Local => {
-                    self.browser.local_mut().toggle_hidden_files();
+                FileExplorerTab::FindHostBridge | FileExplorerTab::HostBridge => {
+                    self.browser.host_bridge_mut().toggle_hidden_files();
                     self.refresh_local_status_bar();
                     self.update_browser_file_list();
                 }
