@@ -5,7 +5,7 @@
 
 // Locals
 // Ext
-use tuirealm::tui::layout::{Constraint, Direction, Layout};
+use tuirealm::ratatui::layout::{Constraint, Direction, Layout};
 
 use super::{components, Context, Id, IdCommon, IdTheme, SetupActivity, Theme, ViewLayout};
 
@@ -39,7 +39,7 @@ impl SetupActivity {
                     ]
                     .as_ref(),
                 )
-                .split(f.size());
+                .split(f.area());
             // Render common widget
             self.app.view(&Id::Common(IdCommon::Header), f, chunks[0]);
             self.app.view(&Id::Common(IdCommon::Footer), f, chunks[2]);
