@@ -9,13 +9,13 @@ use std::str::FromStr;
 // Ext
 use bytesize::ByteSize;
 use lazy_regex::{Lazy, Regex};
-use tuirealm::tui::style::Color;
+use tuirealm::ratatui::style::Color;
 use tuirealm::utils::parser as tuirealm_parser;
 
 #[cfg(smb)]
 use crate::filetransfer::params::SmbParams;
 use crate::filetransfer::params::{
-    AwsS3Params, GenericProtocolParams, ProtocolParams, WebDAVProtocolParams,
+    AwsS3Params, GenericProtocolParams, KubeProtocolParams, ProtocolParams, WebDAVProtocolParams,
 };
 use crate::filetransfer::{FileTransferParams, FileTransferProtocol};
 #[cfg(not(test))] // NOTE: don't use configuration during tests
@@ -42,7 +42,7 @@ static REMOTE_OPT_PROTOCOL_REGEX: Lazy<Regex> = lazy_regex!(r"(?:([a-z0-9]+)://)
  *  - group 4: Some(path) | None
  */
 static REMOTE_GENERIC_OPT_REGEX: Lazy<Regex> = lazy_regex!(
-    r"(?:([^@]+)@)?(?:([^:]+))(?::((?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])(?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])))?(?::([^:]+))?"
+    r"(?:(.+[^@])@)?(?:([^:]+))(?::((?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])(?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])))?(?::([^:]+))?"
 );
 
 /**
@@ -53,7 +53,15 @@ static REMOTE_GENERIC_OPT_REGEX: Lazy<Regex> = lazy_regex!(
  *  - group 4: Some(path) | None
  */
 static REMOTE_WEBDAV_OPT_REGEX: Lazy<Regex> =
-    lazy_regex!(r"(?:([^:]+):)(?:([^@]+)@)(?:([^/]+))(?:/(.+))?");
+    lazy_regex!(r"(?:([^:]+):)(?:(.+[^@])@)(?:([^/]+))(?:(.+))?");
+
+/**
+ * Regex matches: {namespace}[@{cluster_url}]$/{path}
+ *  - group 1: Namespace
+ *  - group 3: Some(cluster_url) | None
+ *  - group 5: Some(path) | None
+ */
+static REMOTE_KUBE_OPT_REGEX: Lazy<Regex> = lazy_regex!(r"(?:([^@]+))(@(?:([^$]+)))?(\$(?:(.+)))?");
 
 /**
  * Regex matches:
@@ -63,7 +71,7 @@ static REMOTE_WEBDAV_OPT_REGEX: Lazy<Regex> =
  * - group 4: Some(path) | None
  */
 static REMOTE_S3_OPT_REGEX: Lazy<Regex> =
-    lazy_regex!(r"(?:([^@]+)@)(?:([^:]+))(?::([a-zA-Z0-9][^:]+))?(?::([^:]+))?");
+    lazy_regex!(r"(?:(.+[^@])@)(?:([^:]+))(?::([a-zA-Z0-9][^:]+))?(?::([^:]+))?");
 
 /**
  * Regex matches:
@@ -75,7 +83,7 @@ static REMOTE_S3_OPT_REGEX: Lazy<Regex> =
  */
 #[cfg(smb_unix)]
 static REMOTE_SMB_OPT_REGEX: Lazy<Regex> = lazy_regex!(
-    r"(?:([^@]+)@)?(?:([^/:]+))(?::((?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])(?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])))?(?:/([^/]+))?(?:(/.+))?"
+    r"(?:(.+[^@])@)?(?:([^/:]+))(?::((?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])(?:[0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])))?(?:/([^/]+))?(?:(/.+))?"
 );
 
 /**
@@ -87,15 +95,14 @@ static REMOTE_SMB_OPT_REGEX: Lazy<Regex> = lazy_regex!(
  */
 #[cfg(smb_windows)]
 static REMOTE_SMB_OPT_REGEX: Lazy<Regex> =
-    lazy_regex!(r"(?:([^@]+)@)?(?:([^:\\]+))(?:\\([^\\]+))?(?:(\\.+))?");
+    lazy_regex!(r"(?:(.+[^@])@)?(?:([^:\\]+))(?:\\([^\\]+))?(?:(\\.+))?");
 
 /**
  * Regex matches:
  * - group 1: Version
- * E.g. termscp-0.3.2 => 0.3.2
- *      v0.4.0 => 0.4.0
+ *      E.g. termscp-0.3.2 => 0.3.2; v0.4.0 => 0.4.0
  */
-static SEMVER_REGEX: Lazy<Regex> = lazy_regex!(r".*(:?[0-9]\.[0-9]\.[0-9])");
+static SEMVER_REGEX: Lazy<Regex> = lazy_regex!(r"v?((0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*))");
 
 /**
  * Regex matches:
@@ -162,6 +169,7 @@ pub fn parse_remote_opt(s: &str) -> Result<FileTransferParams, String> {
     // Match against regex for protocol type
     match protocol {
         FileTransferProtocol::AwsS3 => parse_s3_remote_opt(remote.as_str()),
+        FileTransferProtocol::Kube => parse_kube_remote_opt(remote.as_str()),
         #[cfg(smb)]
         FileTransferProtocol::Smb => parse_smb_remote_opts(remote.as_str()),
         FileTransferProtocol::WebDAV => {
@@ -302,6 +310,29 @@ fn parse_s3_remote_opt(s: &str) -> Result<FileTransferParams, String> {
     }
 }
 
+fn parse_kube_remote_opt(s: &str) -> Result<FileTransferParams, String> {
+    match REMOTE_KUBE_OPT_REGEX.captures(s) {
+        Some(groups) => {
+            let namespace: Option<String> = groups.get(1).map(|x| x.as_str().to_string());
+            let cluster_url: Option<String> = groups.get(3).map(|x| x.as_str().to_string());
+            let remote_path: Option<PathBuf> =
+                groups.get(5).map(|group| PathBuf::from(group.as_str()));
+            Ok(FileTransferParams::new(
+                FileTransferProtocol::Kube,
+                ProtocolParams::Kube(KubeProtocolParams {
+                    namespace,
+                    cluster_url,
+                    username: None,
+                    client_cert: None,
+                    client_key: None,
+                }),
+            )
+            .remote_path(remote_path))
+        }
+        None => Err(String::from("Bad remote host syntax!")),
+    }
+}
+
 /// Parse remote options for smb protocol
 #[cfg(smb_unix)]
 fn parse_smb_remote_opts(s: &str) -> Result<FileTransferParams, String> {
@@ -596,6 +627,21 @@ mod tests {
         assert!(parse_remote_opt(&String::from("omar://172.26.104.1")).is_err());
         // Bad port
         assert!(parse_remote_opt(&String::from("scp://172.26.104.1:650000")).is_err());
+
+        // with @ in username
+        let result: FileTransferParams =
+            parse_remote_opt(&String::from("dummy@veeso.dev@172.26.104.1:8022"))
+                .ok()
+                .unwrap();
+        let params = result.params.generic_params().unwrap();
+        assert_eq!(params.address, String::from("172.26.104.1"));
+        assert_eq!(params.port, 8022);
+        assert_eq!(
+            params.username.as_deref().unwrap().to_string(),
+            String::from("dummy@veeso.dev")
+        );
+        assert_eq!(result.protocol, FileTransferProtocol::Sftp);
+        assert!(result.remote_path.is_none());
     }
 
     #[test]
@@ -615,6 +661,24 @@ mod tests {
         assert_eq!(params.uri.as_str(), "http://myserver:4445");
         assert_eq!(params.username.as_str(), "omar");
         assert_eq!(params.password.as_str(), "password");
+
+        // remote path
+        assert_eq!(
+            result.remote_path.unwrap(),
+            PathBuf::from("/myshare/dir/subdir")
+        );
+    }
+
+    #[test]
+    fn test_should_parse_webdav_opt_with_at() {
+        let result =
+            parse_remote_opt("https://omar@veeso.dev:password@myserver:4445/myshare/dir/subdir")
+                .unwrap();
+
+        let params = result.params.webdav_params().unwrap();
+        assert_eq!(params.uri.as_str(), "https://myserver:4445");
+        assert_eq!(params.username.as_str(), "omar@veeso.dev");
+        assert_eq!(params.password.as_str(), "password");
     }
 
     #[test]
@@ -665,6 +729,36 @@ mod tests {
         assert_eq!(params.profile.as_deref(), Some("default"));
         // -- bad args
         assert!(parse_remote_opt(&String::from("s3://mybucket:default:/foobar")).is_err());
+
+        // with @
+        let result: FileTransferParams = parse_remote_opt(&String::from(
+            "s3://omar@mybucket@eu-central-1:default:/foobar",
+        ))
+        .ok()
+        .unwrap();
+        let params = result.params.s3_params().unwrap();
+        assert_eq!(result.protocol, FileTransferProtocol::AwsS3);
+        assert_eq!(result.remote_path, Some(PathBuf::from("/foobar")));
+        assert_eq!(params.bucket_name.as_str(), "omar@mybucket");
+        assert_eq!(params.region.as_deref().unwrap(), "eu-central-1");
+    }
+
+    #[test]
+    fn should_parse_kube_address() {
+        let result = parse_remote_opt("kube://my-namespace@http://localhost:1234$/tmp")
+            .ok()
+            .unwrap();
+        let params = result.params.kube_params().unwrap();
+
+        assert_eq!(params.namespace, Some("my-namespace".to_string()));
+        assert_eq!(params.cluster_url.as_deref(), Some("http://localhost:1234"));
+        assert_eq!(params.username, None);
+        assert_eq!(params.client_cert, None);
+        assert_eq!(params.client_key, None);
+        assert_eq!(
+            result.remote_path.as_deref().unwrap(),
+            std::path::Path::new("/tmp")
+        );
     }
 
     #[test]
@@ -741,6 +835,8 @@ mod tests {
         assert_eq!(parse_semver("v0.4.1").unwrap(), String::from("0.4.1"),);
         assert_eq!(parse_semver("1.0.0").unwrap(), String::from("1.0.0"),);
         assert!(parse_semver("v1.1").is_none());
+
+        assert_eq!(parse_semver("10.15.10"), Some("10.15.10".to_string()));
     }
 
     #[test]
